/*
 *      Program do przegladania logow z ekg.
 *      Klasa do scalania logow.
 *
 *
 *      copyright (c) 2003 by Lukasz Bartnik
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "merge_obj.h"

void Merge::add(type_t type, string path)
{
	_paths.push_back(path);
	_types.push_back(type);
}


void Merge::add(string line)
{
	string type = line.substr(0, line.find(" ", 0));

	if (type == "dir")
		_types.push_back(DIR);
	else if (type == "file")
		_types.push_back(FILE);
	else if (type == "dirgz")
		_types.push_back(DIRGZ);
	else if (type == "filegz")
		_types.push_back(FILEGZ);
	else {
		addError(MERGE_WRONG_LINE_TYPE + line);
		return;
	}

	_paths.push_back(line.substr(line.find(" ", 0)+1, string::npos));
}

bool Merge::merge()
{
	unsigned int i=0;
	bool merge_state, whole_state = true;
	EkgLog::dBase::log_format_t logfmt;
	string error_str;

	// zerujemy statystyki
	_messages = _db.messages();
	_statuses = _db.statuses();
	

	check_paths();
	_errors.clear();
	
	for ( ;i<_types.size(); ++i) {
		switch (_types[i]) {
			case DIR:	logfmt = EkgLog::dBase::DIR;	break;
			case DIRGZ:	logfmt = EkgLog::dBase::DIRGZ;	break;
			case FILE:	logfmt = EkgLog::dBase::FILE;	break;
			case FILEGZ:	logfmt = EkgLog::dBase::FILEGZ;	break;
		}

		merge_state = true;
		try {
			merge_state = _db.merge(_paths[i], logfmt);
		} catch (std::runtime_error & e) {
			addError(MERGE_RUNTIME_ERROR_P1 + _paths[i] +
				     MERGE_RUNTIME_ERROR_P2 + e.what());
			whole_state = false;
		}

		if (!merge_state) {
			whole_state = false;
			addError(_db.get_error());
		}
	}

	// obliczamy wczytane
	_messages = _db.messages() - _messages;
	_statuses = _db.statuses() - _statuses;

	// jesli wczytano cos nowego...
	if (_messages || _statuses)
		env.db_changed(true);

	_stats = MERGE_STATS_1 + text_cast(_messages) + MERGE_STATS_2 +
		 text_cast(_statuses) + MERGE_STATS_3;

	_paths.clear();
	_types.clear();

	return whole_state;
}


void Merge::check_paths()
{
	const char * home = getenv("HOME");
	for (vector<string>::iterator i=_paths.begin(); i!=_paths.end(); ++i) {
		if ( (*i)[0] == '~' )
			i->replace(0, 1, home);
		// jesli plik konczy sie na .gz do najpewniej jest to plik spakowany
		// (trzeba by jeszcze sprawdzac types), a biblioteka sama dodaje te
		// koncowke do pliku - tutaj wiec ja usuwamy
		if ( !(i->substr(i->size()-3, 3).compare(0,3,".gz")) )
			i->erase(i->size()-3, 3);
	}
}


bool Merge::processFile(const string & file)
{
	ifstream in(file.c_str());
	string line;

	while (true) {
		getline(in, line);
		if (!in)
			break;
		add(line);
	}

	return merge();
}

// jesli scalanie bedzie mozliwe z linii komend klienta, to wtedy
// wypisywanie nie bedzie do cout, tylko do okna programu
void Merge::printError(const string & error)
{
	cout << error << endl;
}


void Merge::addError(const string & error)
{
	_errors.push_back(error);
}


const string & Merge::getErrors()
{
	static string errorsLine;

	for (list<string>::iterator i=_errors.begin(); i!=_errors.end(); ++i)
		errorsLine += *i + '\n';

	if ( errorsLine.size())
		errorsLine.erase(errorsLine.size()-1, 1);

	return errorsLine;
}

