/*
 *      Program do przegladania logow z ekg.
 *      Klasa do scalania logow.
 *
 *
 *      copyright (c) 2003 by Lukasz Bartnik
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __MERGE_OBJ_HH__
#define __MERGE_OBJ_HH__

#include <vector>
#include <string>
#include <cstdlib>
#include <fstream>
#include <iostream>
#include <list>
#include "ekglog.hpp"
#include "texts.h"
#include "environment.h"
#include "stuff.h"

using namespace std;

/** Zbior metod do scalania logow. */
class Merge {
public:
	/** Jako jedyny argument referencja do bazy danych logow. */
	Merge(EkgLog::dBase & db) : _db(db) {}

	// Typy logow
	enum type_t { FILE, DIR, FILEGZ, DIRGZ };

	// dodaj plik do listy do scalenia
	void add(type_t type, string path);

	/** Wyjatek rzucany przez add(string line) */
	struct wrong_entry {
			wrong_entry(const string & s) : _str(s) {}
			const string & get_error() { return _str; }
		private:
			string _str;
	};

	/** Zinterpretuj linie, a potem dopisz plik do dodania.
	 * W razie blednego wpisu w stringu rzuca wyjatkiem. */
	void add(string line);

	/** Wykonaj scalanie wpisanych do tej pory logow.
	 * W przypadku bledu(ow) zwraca false. Przy sukcesie true. */
	bool merge();

	/** Zwraca tresc bledu, ktory moze wystapic przy scalaniu */
	const string & getErrors();
	/** Dodaj wpis o bledzie */
	void addError(const string & error);

	/** Otwiera podany plik i wczytuje: typ sciezka, po czym
	 * scala tak podane logi. */
	bool processFile(const string & file);

	/** Zwraca false jesli ostatnie scalanie spowodowalo bledy */
	operator bool () { return !_errors.empty(); }

	/** Statystyki wczytanych wiadomosci i statusow */
	unsigned int messages() const { return _messages; }
	unsigned int statuses() const { return _statuses; }

	/** Statystyki w tekscie */
	const string & stats() const { return _stats; }

private:
	// podmienia znaki ~ na $HOME
	void check_paths();
	// wypisuje informacje o bledzie
	void printError(const string & error);

	// referencja do bazy danych
	EkgLog::dBase & _db;
	// lista sciezek z logami
	vector<string> _paths;
	// lista typow logow odpowiadajaca _paths
	vector<type_t> _types;
	// informacja o ew. bledzie przy scalaniu
	list<string> _errors;
	// dla statystyk
	unsigned int _messages, _statuses;
	// napis ze statystykami
	string _stats;
	
};


#endif /* __MERGE_HH__ */
