/*
 *      Program do przegladania logow z ekg.
 *      Interfejs uzytkownika.
 *
 *      copyright (c) 2003 by Lukasz Bartnik
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __UI_HH__
#define __UI_HH__

#include "ui-basic.h"
#include "command-base.h"

#include "winlist.h"
#include "winmain.h"
#include "cmdline.h"
#include "environment.h"
#include "list.h"

#include "ekglog.hpp"
using namespace EkgLog;

#include <string>
using namespace std;

/**
 * @brief Interfejs uzytkownika.
 *
 * Calosc interakcji z uzytkownikiem, zawiera w sobie wszystkie okna,
 * linie polecen, mape zarejestrowanych komend etc.
 */
class ui_t : public ui_basic_t {
	friend class command_base_t;
public:

	ui_t(dBase & db);
	~ui_t();

	/// glowna petla programu
	void loop(bool db_good);
	/// dodaje do obslugi komende
	void add_cmd(const std::string & cmd, const std::vector<std::string> & argv)
	{ _runtime_cmd.push_back(cmd_t(cmd, argv)); }
	/// dodanie tekst do wyswietlenia przy starcie ui - info
	void addOpeningInfoText(const string & str) { _opening_info_text += str; }
	/// dodanie tekst do wyswietlenia przy starcie ui - bledy
	void addOpeningErrorText(const string & str) { _opening_error_text += str; }
	/// Wypisanie tekstu do okna debugowego
	void debugPrint(const string & str);
	/// odswieza dopelnianie statystyk (przy zmianie zaw. listy nickow)
	void reloadStats() { _cmdList->prepareCompl(); }

private:

	/// odswieza zawartosc ekranu
	void refresh();
	/// Ustawia zawartosc list.
	void set_lists();

	/// wymiary okien
	unsigned int _list_width,_main_width,_main_height,
		     _nicks_height,_chats_height;
	/// okna ncurses dla obiektow-okien ui
	WINDOW * nc_main, *nc_nicks, *nc_chats, *nc_cmdline, *nc_status;
	/// dwa okna glowne - do wyswietlania rozmow i komunikatow
	winmain_t _main, _info;
#ifndef NDEBUG
	winmain_t _debug;
#endif
	/// dwie listy wyboru - kontakty i rozmowy
	winlist_t _nicks, _chats;
	/// linia polecen
	cmdline_t _cmdline;
	/// pasek stanu
	winstatus_t _status;
	/// mapa z komendami interfejsu z pliku ui-commands.h
	std::map<std::string, command_base_t* > _cmds;
	/// ustawiajac to na true, konczy sie program
	bool _end_program;
	/// wskaznik na aktywne okno - jesli aktywna linia komend, to 0
	window_t * _active_window;

	/// referencja do bazy danych
	dBase & _db;

	/// komenda do wykonania
	struct cmd_t {
		cmd_t(const std::string & c, const std::vector<std::string> & a)
			: cmd(c), argv(a) {}
	
		std::string cmd;
		std::vector<std::string> argv;
	};

	/// wektor komend to uzytku w czasie pracy programu
	std::list<cmd_t> _runtime_cmd;
	/// tekst do wyswietlenie przy starcie ui
	string _opening_info_text, _opening_error_text;
	/// komenda statystyk
	cmd_list_t * _cmdList;
};

#endif /* __UI_HH__ */
